@@ -265,12 +265,7 @@ xfce_appfinder_model_init (XfceAppfinderModel *model)
 
   model->menu = garcon_menu_new_applications ();
   appfinder_refcount_debug_add (G_OBJECT (model->menu), "main menu");
-
-#if GLIB_CHECK_VERSION (2, 32, 0)
   model->collect_thread = g_thread_new ("Collector", xfce_appfinder_model_collect_thread, model);
-#else
-  model->collect_thread = g_thread_create (xfce_appfinder_model_collect_thread, model, TRUE, NULL);
-#endif
 }
 
 
@@ -1749,8 +1744,6 @@ xfce_appfinder_model_menu_changed_idle (gpointer data)
 
   APPFINDER_DEBUG ("menu changed");
 
-  model->menu_changed_idle_id = 0;
-
   /* add all the old items in a garcon database */
   old_items = g_hash_table_new (g_str_hash, g_str_equal);
   for (li = model->items; li != NULL; li = li->next)
@@ -1767,6 +1760,10 @@ xfce_appfinder_model_menu_changed_idle (gpointer data)
     {
       g_warning ("Failed to reload the root menu: %s", error->message);
       g_error_free (error);
+
+      model->menu_changed_idle_id = 0;
+
+      return FALSE;
     }
 
   xfce_appfinder_model_collect_menu (menu, NULL, &collect_items, &collect_categories);
@@ -1856,6 +1853,8 @@ xfce_appfinder_model_menu_changed_idle (gpointer data)
   g_slist_foreach (tmp, (GFunc) (void (*)(void)) g_object_unref, NULL);
   g_slist_free (tmp);
 
+  model->menu_changed_idle_id = 0;
+
   return FALSE;
 }
 
@@ -1870,7 +1869,7 @@ xfce_appfinder_model_menu_changed (GarconMenu         *menu,
   appfinder_return_if_fail (model->menu == menu);
 
   if (model->menu_changed_idle_id == 0)
-    model->menu_changed_idle_id = g_idle_add (xfce_appfinder_model_menu_changed_idle, model);
+    model->menu_changed_idle_id = g_timeout_add (1000, xfce_appfinder_model_menu_changed_idle, model);
 }
 
 
@@ -2369,10 +2368,10 @@ xfce_appfinder_model_execute (XfceAppfinderModel  *model,
                               gboolean            *is_regular_command,
                               GError             **error)
 {
-  const gchar     *command, *p;
+  const gchar     *icon;
+  gchar           *command, *uri;
   GarconMenuItem  *item;
   ModelItem       *mitem;
-  GString         *string;
   gboolean         succeed = FALSE;
   gchar          **argv;
 
@@ -2390,51 +2389,35 @@ xfce_appfinder_model_execute (XfceAppfinderModel  *model,
 
   appfinder_return_val_if_fail (GARCON_IS_MENU_ITEM (item), FALSE);
 
-  command = garcon_menu_item_get_command (item);
+  command = (gchar*) garcon_menu_item_get_command (item);
   if (!IS_STRING (command))
     {
       g_set_error_literal (error, 0, 0, _("Application has no command"));
       return FALSE;
     }
 
-  string = g_string_sized_new (100);
-
-  if (garcon_menu_item_requires_terminal (item))
-    g_string_append (string, "exo-open --launch TerminalEmulator ");
-
   /* expand the field codes */
-  for (p = command; *p != '\0'; ++p)
-    {
-      if (G_UNLIKELY (p[0] == '%' && p[1] != '\0'))
-        {
-          switch (*++p)
-            {
-            case '%':
-              g_string_append_c (string, '%');
-              break;
+  icon = garcon_menu_item_get_icon_name (item);
+  uri = garcon_menu_item_get_uri (item);
+  command = xfce_expand_desktop_entry_field_codes (command, NULL, icon,
+                                                   garcon_menu_item_get_name (item),
+                                                   uri,
+                                                   garcon_menu_item_requires_terminal (item));
+  g_free (uri);
 
-            /* skip all the other %? values for now we don't have dnd anyways */
-            }
-        }
-      else
-        {
-          g_string_append_c (string, *p);
-        }
-    }
-
-  if (g_shell_parse_argv (string->str, NULL, &argv, error))
+  if (g_shell_parse_argv (command, NULL, &argv, error))
     {
-      succeed = xfce_spawn_on_screen (screen, garcon_menu_item_get_path (item),
+      succeed = xfce_spawn_on_screen (screen,
+                                      garcon_menu_item_get_path (item),
                                       argv, NULL, G_SPAWN_SEARCH_PATH,
                                       garcon_menu_item_supports_startup_notification (item),
                                       gtk_get_current_event_time (),
-                                      garcon_menu_item_get_icon_name (item),
-                                      error);
+                                      icon, error);
 
       g_strfreev (argv);
     }
 
-  g_string_free (string, TRUE);
+  g_free (command);
 
   return succeed;
 }
