@@ -1037,21 +1037,32 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
       uri_is_local = g_str_has_prefix (uri, XFCE_APPFINDER_LOCAL_PREFIX);
 
       menu = gtk_menu_new ();
+      gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
       g_object_set_data_full (G_OBJECT (menu), "uri", uri, g_free);
       g_object_set_data_full (G_OBJECT (menu), "name", title, g_free);
       g_object_set_data (G_OBJECT (menu), "model", model);
       g_signal_connect (G_OBJECT (menu), "selection-done",
           G_CALLBACK (gtk_widget_destroy), NULL);
 
-      mi = gtk_menu_item_new_with_label (title);
+      /* Title */
+      mi = gtk_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_set_sensitive (mi, FALSE);
-      gtk_widget_show (mi);
 
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+      label = gtk_label_new (title);
+      image = gtk_image_new_from_icon_name ("", GTK_ICON_SIZE_MENU);
+      gtk_container_add (GTK_CONTAINER (box), image);
+      gtk_container_add (GTK_CONTAINER (box), label);
+      gtk_container_add (GTK_CONTAINER (mi), box);
+      gtk_widget_show_all (mi);
+
+      /* Separator */
       mi = gtk_separator_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_show (mi);
 
+      /* Add/Remove from Bookmarks */
       mi = gtk_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
@@ -1069,10 +1080,12 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
       gtk_container_add (GTK_CONTAINER (mi), box);
       gtk_widget_show_all (mi);
 
+      /* Separator */
       mi = gtk_separator_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_show (mi);
 
+      /* Launch */
       mi = gtk_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
@@ -1086,6 +1099,7 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
       gtk_container_add (GTK_CONTAINER (mi), box);
       gtk_widget_show_all (mi);
 
+      /* Edit */
       mi = gtk_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
@@ -1099,13 +1113,13 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
       gtk_container_add (GTK_CONTAINER (mi), box);
       gtk_widget_show_all (mi);
 
+      /* Revert */
       mi = gtk_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
           G_CALLBACK (xfce_appfinder_window_popup_menu_revert), window);
       path = xfce_resource_save_location (XFCE_RESOURCE_DATA, "applications/", FALSE);
       gtk_widget_set_sensitive (mi, uri_is_local && g_str_has_prefix (uri + 7, path));
-      gtk_widget_show (mi);
       g_free (path);
 
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
@@ -1116,12 +1130,20 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
       gtk_container_add (GTK_CONTAINER (mi), box);
       gtk_widget_show_all (mi);
 
-      mi = gtk_menu_item_new_with_mnemonic (_("_Hide"));
+      /* Hide */
+      mi = gtk_menu_item_new ();
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_set_sensitive (mi, uri_is_local);
       g_signal_connect (G_OBJECT (mi), "activate",
           G_CALLBACK (xfce_appfinder_window_popup_menu_hide), window);
-      gtk_widget_show (mi);
+
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+      label = gtk_label_new_with_mnemonic (_("_Hide"));
+      image = gtk_image_new_from_icon_name ("", GTK_ICON_SIZE_MENU);
+      gtk_container_add (GTK_CONTAINER (box), image);
+      gtk_container_add (GTK_CONTAINER (box), label);
+      gtk_container_add (GTK_CONTAINER (mi), box);
+      gtk_widget_show_all (mi);
 
 #if GTK_CHECK_VERSION (3, 22, 0)
       gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
@@ -1266,6 +1288,10 @@ xfce_appfinder_window_entry_changed_idle (gpointer data)
     {
       gtk_widget_set_sensitive (window->button_launch, IS_STRING (text));
 
+      /* Clear entry's error icon and tooltip */
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY, NULL);
+      gtk_entry_set_icon_tooltip_text (GTK_ENTRY (window->entry), GTK_ENTRY_ICON_PRIMARY, NULL);
+
       pixbuf = xfce_appfinder_model_get_icon_for_command (window->model, text);
       xfce_appfinder_window_update_image (window, pixbuf);
       if (pixbuf != NULL)
@@ -1983,11 +2009,13 @@ xfce_appfinder_window_sort_items (GtkTreeModel *model,
 
   gtk_tree_model_get (model, a, XFCE_APPFINDER_MODEL_COLUMN_TITLE, &title_a, -1);
   normalized = g_utf8_normalize (title_a, -1, G_NORMALIZE_ALL);
+  g_free (title_a);
   title_a = g_utf8_casefold (normalized, -1);
   g_free (normalized);
 
   gtk_tree_model_get (model, b, XFCE_APPFINDER_MODEL_COLUMN_TITLE, &title_b, -1);
   normalized = g_utf8_normalize (title_b, -1, G_NORMALIZE_ALL);
+  g_free (title_b);
   title_b = g_utf8_casefold (normalized, -1);
   g_free (normalized);
 
